@@ -29,11 +29,13 @@
 
 from __future__ import division
 
-__version__ = '0.1'
+__version__ = '0.3'
 __all__ = ['Flavor', 'Table', 'Literal', 'Column', 'Join', 'Asc', 'Desc']
 
 import string
+from functools import partial
 from threading import local, currentThread
+from collections import defaultdict
 
 
 def alias(i, letters=string.ascii_lowercase):
@@ -108,7 +110,7 @@ class AliasManager(object):
     '''
     Context Manager for unique alias generation
     '''
-    __slots__ = 'local'
+    __slots__ = ()
 
     local = local()
     local.alias = None
@@ -117,7 +119,7 @@ class AliasManager(object):
     @classmethod
     def __enter__(cls):
         if getattr(cls.local, 'alias', None) is None:
-            cls.local.alias = {}
+            cls.local.alias = defaultdict(cls.alias_factory)
             cls.local.nested = 0
         cls.local.nested += 1
 
@@ -131,10 +133,18 @@ class AliasManager(object):
     def get(cls, from_):
         if getattr(cls.local, 'alias', None) is None:
             return ''
-        i = len(cls.local.alias)
-        cls.local.alias.setdefault(from_, alias(i))
         return cls.local.alias[from_]
 
+    @classmethod
+    def set(cls, from_, alias):
+        assert cls.local.alias.get(from_) is None
+        cls.local.alias[from_] = alias
+
+    @classmethod
+    def alias_factory(cls):
+        i = len(cls.local.alias)
+        return alias(i)
+
 
 class Query(object):
     __slots__ = ()
@@ -346,23 +356,23 @@ class Select(Query, FromItem, _SelectQueryMixin):
 
     @property
     def params(self):
-        p = ()
+        p = []
         for column in self.columns:
             if isinstance(column, As):
-                p += column.expression.params
-            p += column.params
-        p += self.from_.params
+                p.extend(column.expression.params)
+            p.extend(column.params)
+        p.extend(self.from_.params)
         if self.where:
-            p += self.where.params
+            p.extend(self.where.params)
         if self.group_by:
             for expression in self.group_by:
-                p += expression.params
+                p.extend(expression.params)
         if self.having:
-            p += self.having.params
+            p.extend(self.having.params)
         if self.order_by:
             for expression in self.order_by:
-                p += expression.params
-        return p
+                p.extend(expression.params)
+        return tuple(p)
 
     def __or__(self, other):
         return Union(self, other)
@@ -428,13 +438,15 @@ class Insert(Query):
         self.__returning = value
 
     @staticmethod
-    def _format(value):
+    def _format(value, param=None):
+        if param is None:
+            param = Flavor.get().param
         if isinstance(value, Expression):
             return str(value)
         elif isinstance(value, Select):
             return '(%s)' % value
         else:
-            return Flavor.get().param
+            return param
 
     def __str__(self):
         columns = ''
@@ -442,8 +454,9 @@ class Insert(Query):
             assert all(col.table == self.table for col in self.columns)
             columns = ' (' + ', '.join(map(str, self.columns)) + ')'
         if isinstance(self.values, list):
+            format_ = partial(self._format, param=Flavor.get().param)
             values = ' VALUES ' + ', '.join(
-                '(' + ', '.join(map(self._format, v)) + ')'
+                '(' + ', '.join(map(format_, v)) + ')'
                 for v in self.values)
             # TODO manage DEFAULT
         elif isinstance(self.values, Select):
@@ -457,28 +470,29 @@ class Insert(Query):
 
     @property
     def params(self):
+        p = []
         if isinstance(self.values, list):
-            p = ()
             for values in self.values:
                 for value in values:
-                    if isinstance(value, Expression):
-                        p += value.params
-                    elif isinstance(value, Select):
-                        p += list(value)[1]
+                    if isinstance(value, (Expression, Select)):
+                        p.extend(value.params)
                     else:
-                        p += (value,)
-            return p
+                        p.append(value)
         elif isinstance(self.values, Select):
-            return list(self.values)[1]
-        else:
-            return ()
+            p.extend(self.values.params)
+        if self.returning:
+            for exp in self.returning:
+                p.extend(exp.params)
+        return tuple(p)
 
 
 class Update(Insert):
     __slots__ = ('__where', '__values', 'from_')
 
-    def __init__(self, table, columns, values, from_=None, where=None):
-        super(Update, self).__init__(table, columns=columns, values=values)
+    def __init__(self, table, columns, values, from_=None, where=None,
+            returning=None):
+        super(Update, self).__init__(table, columns=columns, values=values,
+            returning=returning)
         self.__where = None
         self.from_ = From(from_) if from_ else None
         self.where = where
@@ -510,38 +524,41 @@ class Update(Insert):
         # Get columns without alias
         columns = map(str, self.columns)
 
-        def format():
+        with AliasManager():
             from_ = ''
             if self.from_:
+                table = From([self.table])
                 from_ = ' FROM %s' % str(self.from_)
+            else:
+                table = self.table
+                AliasManager.set(table, str(table)[1:-1])
             values = ', '.join('%s = %s' % (c, self._format(v))
                 for c, v in zip(columns, self.values))
             where = ''
             if self.where:
                 where = ' WHERE ' + str(self.where)
-            return ('UPDATE %s SET ' % From([self.table]) + values + from_ +
-                where)
-        if self.from_:
-            with AliasManager():
-                return format()
-        else:
-            return format()
+            returning = ''
+            if self.returning:
+                returning = ' RETURNING ' + ', '.join(map(str, self.returning))
+            return ('UPDATE %s SET ' % table + values + from_ + where
+                + returning)
 
     @property
     def params(self):
-        p = ()
+        p = []
         for value in self.values:
-            if isinstance(value, Expression):
-                p += value.params
-            elif isinstance(value, Select):
-                p += list(value)[1]
+            if isinstance(value, (Expression, Select)):
+                p.extend(value.params)
             else:
-                p += (value,)
+                p.append(value)
         if self.from_:
-            p += self.from_.params
+            p.extend(self.from_.params)
         if self.where:
-            p += self.where.params
-        return p
+            p.extend(self.where.params)
+        if self.returning:
+            for exp in self.returning:
+                p.extend(exp.params)
+        return tuple(p)
 
 
 class Delete(Query):
@@ -600,7 +617,13 @@ class Delete(Query):
 
     @property
     def params(self):
-        return self.where.params if self.where else ()
+        p = []
+        if self.where:
+            p.extend(self.where.params)
+        if self.returning:
+            for exp in self.returning:
+                p.extend(exp.params)
+        return tuple(p)
 
 
 class CombiningQuery(Query, FromItem, _SelectQueryMixin):
@@ -621,11 +644,13 @@ class CombiningQuery(Query, FromItem, _SelectQueryMixin):
 
     @property
     def params(self):
-        p = sum((q.params for q in self.queries), ())
+        p = []
+        for q in self.queries:
+            p.extend(q.params)
         if self.order_by:
             for expression in self.order_by:
-                p += expression.params
-        return p
+                p.extend(expression.params)
+        return tuple(p)
 
 
 class Union(CombiningQuery):
@@ -661,9 +686,9 @@ class Table(FromItem):
         return Insert(self, columns=columns, values=values,
             returning=returning)
 
-    def update(self, columns, values, from_=None, where=None):
+    def update(self, columns, values, from_=None, where=None, returning=None):
         return Update(self, columns=columns, values=values, from_=from_,
-            where=where)
+            where=where, returning=returning)
 
     def delete(self, only=False, using=None, where=None, returning=None):
         return Delete(self, only=only, using=using, where=where,
@@ -734,13 +759,13 @@ class Join(FromItem):
 
     @property
     def params(self):
-        p = ()
+        p = []
         for item in (self.left, self.right):
             if hasattr(item, 'params'):
-                p += item.params
+                p.extend(item.params)
         if hasattr(self.condition, 'params'):
-            p += self.condition.params
-        return p
+            p.extend(self.condition.params)
+        return tuple(p)
 
     @property
     def alias(self):
@@ -763,15 +788,27 @@ class From(list):
         def format(from_):
             if isinstance(from_, Query):
                 return '(%s) AS "%s"' % (from_, from_.alias)
-            elif getattr(from_, 'alias', None):
-                return '%s AS "%s"' % (from_, from_.alias)
             else:
-                return str(from_)
+                alias = getattr(from_, 'alias', None)
+                columns_definitions = getattr(from_, 'columns_definitions',
+                    None)
+                # XXX find a better test for __getattr__ which returns Column
+                if (alias and columns_definitions
+                        and not isinstance(columns_definitions, Column)):
+                    return '%s AS "%s" (%s)' % (from_, alias,
+                        columns_definitions)
+                elif alias:
+                    return '%s AS "%s"' % (from_, alias)
+                else:
+                    return str(from_)
         return ', '.join(map(format, self))
 
     @property
     def params(self):
-        return sum((from_.params for from_ in self), ())
+        p = []
+        for from_ in self:
+            p.extend(from_.params)
+        return tuple(p)
 
     def __add__(self, other):
         assert isinstance(other, (Join, Select))
@@ -812,9 +849,13 @@ class Expression(object):
         from sql.operators import Mul
         return Mul(self, other)
 
+    def __div__(self, other):
+        from sql.operators import Div
+        return Div(self, other)
+
     def __floordiv__(self, other):
-        from sql.operators import FloorDiv
-        return FloorDiv(self, other)
+        from sql.functions import Div
+        return Div(self, other)
 
     def __mod__(self, other):
         from sql.operators import Mod
@@ -938,9 +979,10 @@ class Column(Expression):
             t = '%s'
         else:
             t = '"%s"'
-        if self.__from.alias:
+        alias = self.__from.alias
+        if alias:
             t = '"%s".' + t
-            return t % (self.__from.alias, self.__name)
+            return t % (alias, self.__name)
         else:
             return t % self.__name
 
@@ -974,11 +1016,10 @@ class Cast(Expression):
         self.typename = typename
 
     def __str__(self):
-        param = Flavor.get().param
         if isinstance(self.expression, Expression):
             value = self.expression
         else:
-            value = param
+            value = Flavor.get().param
         return 'CAST(%s AS %s)' % (value, self.typename)
 
     @property
